package com.bazaarvoice.hybris.utils;

import com.bazaarvoice.hybris.CronjobType;
import com.bazaarvoice.hybris.constants.BazaarvoiceConstants;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.cms2.model.site.CMSSiteModel;
import de.hybris.platform.cms2.servicelayer.services.CMSSiteService;
import de.hybris.platform.core.model.media.MediaModel;
import de.hybris.platform.core.model.product.ProductModel;
import de.hybris.platform.util.Utilities;
import de.hybris.platform.variants.model.VariantProductModel;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.zip.GZIPOutputStream;

import static com.bazaarvoice.hybris.utils.BazaarVoiceUtils.ReplaceUnsupportedCharacters;

@Component("bvExportXmlUtils")
public class BVExportXmlUtils {

    private static final String FEED_DIR_PATH = Utilities.getExtensionInfo(BazaarvoiceConstants.EXTENSIONNAME)
            .getExtensionDirectory().getAbsolutePath()
            + File.separator + "resources" + File.separator + "feed";

    private static final String TRANSACTION_DIR_PATH = Utilities.getExtensionInfo(BazaarvoiceConstants.EXTENSIONNAME)
            .getExtensionDirectory().getAbsolutePath()
            + File.separator + "resources" + File.separator + "transaction";

    private static final Logger LOG = Logger.getLogger(BVExportXmlUtils.class);

    @Resource(name = "siteBaseUrlResolutionService")
    private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

    @Resource(name = "cmsSiteService")
    private CMSSiteService cmsSiteService;

    public String gerenatedate()
    {
        final Date date = new Date(); // your date
        final Calendar cal = Calendar.getInstance();
        cal.setTime(date);

        final NumberFormat formatter = NumberFormat.getNumberInstance();
        formatter.setMinimumIntegerDigits(2);
        final String year = "" + cal.get(Calendar.YEAR);
        final String month = formatter.format(cal.get(Calendar.MONTH) + 1);
        final String day = formatter.format(cal.get(Calendar.DAY_OF_MONTH));
        final String hour = formatter.format(cal.get(Calendar.HOUR_OF_DAY));
        final String minute = formatter.format(cal.get(Calendar.MINUTE));
        final String secund = formatter.format(cal.get(Calendar.SECOND));
        final String dateNumber = year + month + day + hour + minute + secund;
        return dateNumber;
    }

    public String gerenateDateWithMillisecond()
    {
        final Date date = new Date(); // your date
        final Calendar cal = Calendar.getInstance();
        cal.setTime(date);

        final NumberFormat formatter = NumberFormat.getNumberInstance();
        formatter.setMinimumIntegerDigits(2);
        final String year = "" + cal.get(Calendar.YEAR);
        final String month = formatter.format(cal.get(Calendar.MONTH) + 1);
        final String day = formatter.format(cal.get(Calendar.DAY_OF_MONTH));
        final String hour = formatter.format(cal.get(Calendar.HOUR_OF_DAY));
        final String minute = formatter.format(cal.get(Calendar.MINUTE));
        final String secund = formatter.format(cal.get(Calendar.SECOND));
        final String millisecond = formatter.format(cal.get(Calendar.MILLISECOND));
        final String dateNumber = year + month + day + hour + minute + secund + millisecond;
        return dateNumber;
    }

    public List<File> compressFilesToGz(final List<File> files, final String archiveDirectory,
                                        final String specificXmlArchiveDirectory, CronjobType cronjobType)
    {

        final List<File> compressedFiles = new ArrayList<>();
        if (files == null || files.isEmpty())
        {
            return null;
        }
        GZIPOutputStream gzos = null;
        File outputFile = null;
        final byte[] buffer = new byte[1024];
        try
        {
            FileInputStream in = null;
            for (final File file : files) {
                File pathToDir;
                if(cronjobType.equals(CronjobType.PRODUCT)) {
                    pathToDir = getPathToFeedDir(archiveDirectory, specificXmlArchiveDirectory);
                } else if (cronjobType.equals(CronjobType.TRANSACTION)){
                    pathToDir = getPathToTransactionDir(archiveDirectory, specificXmlArchiveDirectory);
                } else {
                    throw new IllegalArgumentException("This type of cronjob is not supported");
                }
                outputFile = new File(pathToDir.getAbsolutePath(), file.getName() + ".gz");
                in = new FileInputStream(file.getAbsolutePath());
                gzos = new GZIPOutputStream(new FileOutputStream(outputFile));
                int len;
                while ((len = in.read(buffer)) > 0)
                {
                    gzos.write(buffer, 0, len);
                }
                compressedFiles.add(outputFile);
                gzos.close();
                in.close();
            }

        }
        catch (final IOException ex)
        {
            ex.printStackTrace();
            return null;
        }
        finally
        {
            if (gzos != null)
            {
                try
                {
                    gzos.close();
                }
                catch (final IOException e)
                {
                    e.printStackTrace();
                }
            }
        }
        return compressedFiles;
    }

    public File getPathToFeedDir(String localDirectory, String specificDirectory) {
        final File pathToDir;
        if (specificDirectory != null) {
            pathToDir = new File(specificDirectory);
        } else {
            pathToDir = new File(FEED_DIR_PATH + (localDirectory == null ? "" : localDirectory));
        }
        if (!pathToDir.exists())
        {
            pathToDir.mkdirs();
        }
        return pathToDir;
    }

    public File getPathToTransactionDir(String localDirectory, String specificDirectory) {
        final File pathToDir;
        if (specificDirectory != null) {
            pathToDir = new File(specificDirectory);
        } else {
            pathToDir = new File(TRANSACTION_DIR_PATH + (localDirectory == null ? "" : localDirectory));
        }
        if (!pathToDir.exists())
        {
            pathToDir.mkdirs();
        }
        return pathToDir;
    }

    public String getImageUrl(ProductModel model, CMSSiteModel cmsSiteModel) {
        String imageUrlString = null;
        MediaModel media;
        media = model.getPicture();
        imageUrlString = (media != null) ? siteBaseUrlResolutionService.getMediaUrlForSite(cmsSiteModel, false, media.getURL())
                : null;

		/*
		 * if a variant product does not have an imageUrl - set it's parent's image url
		 */
        if ((imageUrlString == null || imageUrlString.isEmpty()) && model instanceof VariantProductModel)
        {
            ProductModel rootProduct = model;
            while (rootProduct instanceof VariantProductModel) {
                rootProduct = ((VariantProductModel) rootProduct).getBaseProduct();
                if (rootProduct.getPicture() != null)
                {
                    media = rootProduct.getPicture();
                    imageUrlString = (media != null) ? siteBaseUrlResolutionService.getMediaUrlForSite(cmsSiteModel, false,
                            media.getURL()) : null;
                    break;
                }
            }
        }
        return imageUrlString;
    }

    public Integer getPort(String server, String ftpServerPort) {
        if(ftpServerPort==null || ftpServerPort.isEmpty()){
            if(server!=null && !server.isEmpty()) {
                return (server.toLowerCase().substring(0,4).equals("sftp") ? Integer.valueOf(22) : Integer.valueOf(21));
            }
        } else {
            try{
                return Integer.valueOf(ftpServerPort);
            } catch (Exception e) {
                return Integer.valueOf(21);
            }
        }
        return null;
    }

    public List<String> getEans(CMSSiteModel cmsSiteModel, ProductModel model, boolean isFamilyEnabled){
        List<String> eans = new ArrayList<>();
        if (cmsSiteModel.getBvConfig().getAggregateAttributesSyndication()){
            if (isFamilyEnabled){
                // collect eans from each variant and set them to each variant
                ProductModel productModel = model;
                while (productModel instanceof VariantProductModel) {
                    productModel = ((VariantProductModel) productModel).getBaseProduct();
                }
                eans = getEansFromBaseAndFromAllVariants(productModel);
            } else {
                // if it is base product, go through it's variants and set all of EANS to this base product
                if (model.getClass().equals(ProductModel.class)){
                    eans = getEansFromBaseAndFromAllVariants(model);
                }
            }
        } else {
            if (model.getEan() != null) {
                eans.add(StringEscapeUtils.escapeXml(model.getEan()));
            }

        }
        return eans;
    }

    public String getProductExternalId(ProductModel model, CMSSiteModel cmsSiteModel){
        if (cmsSiteModel.getBvConfig().getProductPrefixEnabled()){
            return (model.getCode() == null || model.getCode().isEmpty()) ? null
                    : cmsSiteModel.getUid() + "-" + StringEscapeUtils.escapeXml(ReplaceUnsupportedCharacters(model.getCode()));
        } else {
            return (model.getCode() == null || model.getCode().isEmpty()) ? null
                    : StringEscapeUtils.escapeXml(ReplaceUnsupportedCharacters(model.getCode()));
        }
    }

    public List<String> getUpcs(CMSSiteModel cmsSiteModel, ProductModel model, boolean isFamilyEnabled, String upcMethodName) {
        List<String> upcs = new ArrayList<String>();
        if(upcMethodName!=null && !upcMethodName.isEmpty())
        {
            String upc = null;
            Method method = null;
            try {
                if (cmsSiteModel.getBvConfig().getAggregateAttributesSyndication()){
                    if (isFamilyEnabled){
                        // collect upcs from each variant and set them to each variant
                        ProductModel productModel = model;
                        while (productModel instanceof VariantProductModel) {
                            productModel = ((VariantProductModel) productModel).getBaseProduct();
                        }
                        upcs = getUpcsFromBaseAndFromAllVariants(productModel, upcMethodName);
                    } else {
                        // if it is base product, go through it's variants and set all of upcs to this base product
                        if (model.getClass().equals(ProductModel.class)){
                            upcs = getUpcsFromBaseAndFromAllVariants(model, upcMethodName);
                        }
                    }
                } else {
                    if (model.getClass().equals(ProductModel.class)) {
                        method = model.getClass().getDeclaredMethod("get" + upcMethodName, null);
                        upc = (String) method.invoke(model, null);
                        if(upc!=null)
                            upcs.add(StringEscapeUtils.escapeXml(upc));
                    } else {
                        method = model.getClass().forName("de.hybris.platform.core.model.product.ProductModel").getDeclaredMethod("get" + upcMethodName, null);
                        upc = (String) method.invoke(model, null);
                        if(upc!=null)
                            upcs.add(StringEscapeUtils.escapeXml(upc));
                    }
                }

            } catch (NoSuchMethodException e) {
                LOG.debug("No such method exists. Please ensure that attribute name is spelled correctly. Could not find: get" + upcMethodName);
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                LOG.debug("Invocation Error on: get" + upcMethodName);
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                LOG.debug("Unable to access: get" + upcMethodName);
                e.printStackTrace();
            } catch (ClassNotFoundException e) {
                LOG.debug("Please ensure that the model is part of de.hybris.platform.core.model.product.ProductModel hierarchy.");
                e.printStackTrace();
            } catch (final Exception ex) {
                ex.printStackTrace();
            }
        }
        return upcs;
    }

    public String setupUpcMethodName (String upcMethodName){
        Field[] fields = ProductModel.class.getDeclaredFields();
        boolean flag = false;
        if(upcMethodName!=null && !upcMethodName.isEmpty())
        {
            for(Field f:fields){
                f.setAccessible(true);
                if(f.getName().equalsIgnoreCase(upcMethodName)){
                    try {
                        flag=true;
                        upcMethodName = ((String)ProductModel.class.getField(f.getName()).get(f.getName())).substring(0, 1).toUpperCase() +
                                ((String)ProductModel.class.getField(f.getName()).get(f.getName())).substring(1);
                    } catch (NoSuchFieldException e) {
                        LOG.debug("UPC field not configured correctly. No such field exists. Please check the field. UPC will not print.");
                        e.printStackTrace();
                        upcMethodName=null;
                    } catch (IllegalAccessException e) {
                        LOG.debug("Unable to access UPC attribute. Please check UPC field in config.");
                        e.printStackTrace();
                        upcMethodName=null;
                    }
                    break;
                }
            }
        }
        if(!flag)
        {
            upcMethodName=null;
        }
        return upcMethodName;
    }

    private List<String> getUpcsFromBaseAndFromAllVariants(ProductModel baseProduct, String upcMethodName) throws Exception {
        final List<String> upcs = new ArrayList<>();

        Method method = baseProduct.getClass().forName("de.hybris.platform.core.model.product.ProductModel").getDeclaredMethod("get" + upcMethodName, null);
        String baseProductUpc = (String) method.invoke(baseProduct, null);
        if(baseProductUpc!=null) {
            upcs.add(StringEscapeUtils.escapeXml(baseProductUpc));
        }

        if (baseProduct.getVariants() != null) {
            for (VariantProductModel variant : baseProduct.getVariants()) {
                String variantProductUpc = (String) method.invoke(variant, null);
                if (variantProductUpc != null) {
                    upcs.add(StringEscapeUtils.escapeXml(variantProductUpc));
                }
            }
        }
        return upcs;
    }

    private List<String> getEansFromBaseAndFromAllVariants(ProductModel baseProduct) {
        final List<String> eans = new ArrayList<>();
        if (baseProduct.getEan() != null) {
            eans.add(StringEscapeUtils.escapeXml(baseProduct.getEan()));
        }
        if (baseProduct.getVariants() != null) {
            for (VariantProductModel variant : baseProduct.getVariants()) {
                if (variant.getEan() != null) {
                    eans.add(StringEscapeUtils.escapeXml(variant.getEan()));
                }
            }
        }
        return eans;
    }
}
