/**
 *
 */
package com.bazaarvoice.hybris.xml;

import com.bazaarvoice.hybris.utils.BVExportXmlUtils;
import com.bazaarvoice.hybris.xml.tags.*;
import de.hybris.platform.acceleratorservices.urlencoder.attributes.impl.DefaultLanguageAttributeManager;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.category.model.CategoryModel;
import de.hybris.platform.cms2.model.site.CMSSiteModel;
import de.hybris.platform.cms2lib.model.components.ProductDetailComponentModel;
import de.hybris.platform.commerceservices.url.UrlResolver;
import de.hybris.platform.core.model.media.MediaModel;
import de.hybris.platform.core.model.product.ProductModel;
import de.hybris.platform.variants.model.VariantProductModel;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.log4j.Logger;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import java.io.File;
import java.io.IOException;
import java.io.Writer;

import com.sun.xml.bind.marshaller.CharacterEscapeHandler;

import java.util.*;

import static com.bazaarvoice.hybris.utils.BazaarVoiceUtils.ReplaceUnsupportedCharacters;

/**
 * @author christina romashchenko
 */
@Scope("prototype")
@Component
public class ProductFeedXmlCreator
{
	// constants used to declare families for products (used with hybris product variants)
	private static final String BV_FAMILY_NAME_ATTRIBUTE_ID = "BV_FE_FAMILY";
	private static final String BV_FAMILY_EXPAND = "BV_FE_EXPAND";

	@Resource(name = "categoryModelUrlResolver")
	private UrlResolver<CategoryModel> categoryModelUrlResolver;

	@Resource(name = "productModelUrlResolver")
	private UrlResolver<ProductModel> productModelUrlResolver;

	@Resource(name = "siteBaseUrlResolutionService")
	private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

	@Resource(name = "languageAttributeManager")
	private DefaultLanguageAttributeManager languageAttributeManager;

	@Resource (name = "bvExportXmlUtils")
	private BVExportXmlUtils bvExportXmlUtils;

	private static final Logger LOG = Logger.getLogger(ProductFeedXmlCreator.class.getName());
	//private String dateCounter;
	private int fileCounter = 1;
	private List<CategoryModel> brands;

	//	private String host_url;
	private String store_webroot;
	private CMSSiteModel cmsSiteModel;

    private String upcMethodName;

    public void setUpcMethodName(String upcMethodName) {
        this.upcMethodName = upcMethodName;
    }
	/**
	 * @return the brands
	 */
	public List<CategoryModel> getBrands()
	{
		return brands;
	}

	/**
	 * @param brands
	 *           the brands to set
	 */
	public void setBrands(final List<CategoryModel> brands)
	{
		this.brands = brands;
	}

	public void setCMSSiteModel(final CMSSiteModel cmsSiteModel)
	{
		this.cmsSiteModel = cmsSiteModel;
	}

	public void writeXmlToFile(final File file, final ProductFeed productFeed)
	{

		LOG.debug("===========Marshalling Product Fedd to file " + file.getAbsolutePath() + "===================");

		try
		{
			final JAXBContext jaxbContext = JAXBContext.newInstance(ProductFeed.class);
			final Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
			jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			jaxbMarshaller.setProperty(CharacterEscapeHandler.class.getName(),
					new CharacterEscapeHandler() {
						@Override
						public void escape(char[] ac, int i, int j, boolean flag,
										   Writer writer) throws IOException {
							writer.write(ac, i, j);
						}
					});
			jaxbMarshaller.marshal(productFeed, file);

			
		}
		catch (final JAXBException e)
		{
			LOG.error(e.getStackTrace());
		}

	}


	private CategoryTag getCategoryTag(final CategoryModel model, final Map<Locale, String> locales)
	{
		final CategoryTag resultTag = new CategoryTag();

		/* Required */

		//ExternalId
		if (cmsSiteModel.getBvConfig().getStoreLevelCategoriesEnabled()){
			resultTag.setExternalId((model.getCode()==null || StringEscapeUtils.escapeXml(model.getCode()).isEmpty())? null
					: cmsSiteModel.getUid() + "-" + StringEscapeUtils.escapeXml(ReplaceUnsupportedCharacters(model.getCode())));
		} else {
			resultTag.setExternalId((model.getCode()==null || StringEscapeUtils.escapeXml(model.getCode()).isEmpty())? null
					: StringEscapeUtils.escapeXml(ReplaceUnsupportedCharacters(model.getCode())));
		}

		//Name
		resultTag.setName((model.getName()==null || model.getName().isEmpty()) ? null : model.getName());
		//Localized category names
		final List<LocalizedName> names = new ArrayList<LocalizedName>();
		for (final Locale locale : locales.keySet())
		{
			try
			{
				final String localName = model.getName(locale);
				final String normalizedLocale = locales.get(locale);
				if (localName != null && normalizedLocale != null)
				{
					final LocalizedName lname = new LocalizedName(normalizedLocale, localName);
					names.add(lname);
				}
			}
			catch (final Exception ex)
			{
				//LOG.debug(ex.getMessage());
				continue;
			}

		}
		resultTag.setNames(names.isEmpty() ? null : names);

		//CategoryPageUrl
		//		resultTag.setCategoryPageUrl(normalizeText(host_url + categoryModelUrlResolver.resolve(model)));

		final String categoryPageUrl = siteBaseUrlResolutionService.getWebsiteUrlForSite(cmsSiteModel, false,
				categoryModelUrlResolver.resolve(model));
		resultTag.setCategoryPageUrl((categoryPageUrl==null || categoryPageUrl.isEmpty()) ? null : categoryPageUrl);

		/*
		 * Not required parameters
		 */

		/* CategoryPageUrls */
		//		final List<CMSLinkComponentModel> pageUrls = model.getLinkComponents();
		//		if (pageUrls != null)
		//		{
		//
		//		}

		/* ParentExternalId */
		String parentExternalId = null;
		final List<CategoryModel> supercategories = model.getSupercategories();

		/*
		 * supercategories contain several types of elements: 1. a single CategoryModel type element that is our
		 * supercategory 2. ClassificationClassModel that is for classification system usage. So at the moment we use the
		 * only instance of CategoryModel as a parent Category
		 */
		if (!supercategories.isEmpty()){
			for (final CategoryModel supercategory : supercategories) {
				if (supercategory.getClass().equals(CategoryModel.class)) {
					parentExternalId = StringEscapeUtils.escapeXml(ReplaceUnsupportedCharacters(supercategory.getCode()));
				}

				// TODO: think of using classification categories
			}
		}

        if (cmsSiteModel.getBvConfig().getStoreLevelCategoriesEnabled()){
            if (parentExternalId != null && !(StringEscapeUtils.escapeXml(parentExternalId).isEmpty())){
                if (!(parentExternalId.equals("1") || parentExternalId.contains("/") || parentExternalId.equals("collections"))) {
                    resultTag.setParentExternalId(cmsSiteModel.getUid() + "-" + StringEscapeUtils.escapeXml(parentExternalId));
                }
            }
        } else {
            if (parentExternalId != null && !(StringEscapeUtils.escapeXml(parentExternalId).isEmpty())){
                if (!(parentExternalId.equals("1") || parentExternalId.contains("/") || parentExternalId.equals("collections"))) {
                    resultTag.setParentExternalId(StringEscapeUtils.escapeXml(parentExternalId));
                }
            }
        }

		//ImageUrl
		final MediaModel media = model.getPicture();
		final String imageUrl = (media != null) ? siteBaseUrlResolutionService.getMediaUrlForSite(cmsSiteModel, false,
				media.getDownloadURL()) : null;
		resultTag.setImageUrl((imageUrl==null || imageUrl.isEmpty()) ? null : imageUrl);

		//ImageUrls
		//
		//		List<LocalizedImageUrl> imageUrls = new ArrayList<>();
		//		 resultTag.setImageUrls(imageUrls);

		return resultTag;
	}

	public List<CategoryTag> getCategoryTagsFromCollection(final Collection<CategoryModel> categories,
			final Map<Locale, String> locales) {
		final List<CategoryTag> result = new ArrayList<CategoryTag>();
        if (cmsSiteModel.getBvConfig().getStoreLevelCategoriesEnabled()) {
            addParentCategory(result);
        }

		for (final CategoryModel model : categories) {
			final CategoryTag tag = getCategoryTag(model, locales);
			result.add(tag);
		}
        if (cmsSiteModel.getBvConfig().getStoreLevelCategoriesEnabled()) {
            modifyParentCategory(result);
        }

		return result;
	}

	public List<BrandTag> getBrandTagsFromCollection(final Collection<CategoryModel> brandModels, final Map<Locale, String> locales)
	{
		final List<BrandTag> result = new ArrayList<BrandTag>();
		for (final CategoryModel model : brandModels)
		{
			final BrandTag tag = getBrandTag(model, locales);
			result.add(tag);
		}
		return result;
	}

	public List<ProductTag> getProductTagsFromCollection(final Collection<ProductModel> productModels,
			final Map<Locale, String> locales, final boolean isFamilyEnabled)
	{
		final List<ProductTag> result = new ArrayList<ProductTag>();
        upcMethodName = bvExportXmlUtils.setupUpcMethodName(upcMethodName);
		for (final ProductModel model : productModels)
		{
			final ProductTag tag = getProductTag(model, locales, isFamilyEnabled);
			result.add(tag);
		}
		return result;
	}

	private BrandTag getBrandTag(final CategoryModel model, final Map<Locale, String> locales)
	{

		final BrandTag tag = new BrandTag();
		tag.setName((model.getName()==null || model.getName().isEmpty())? null : model.getName());
		tag.setExternalId((model.getCode()==null || StringEscapeUtils.escapeXml(model.getCode()).isEmpty())? null : StringEscapeUtils.escapeXml(ReplaceUnsupportedCharacters(model.getCode())));

		//Localized brand names
		final List<LocalizedName> names = new ArrayList<LocalizedName>();
		for (final Locale locale : locales.keySet())
		{
			try
			{
				final String localName = model.getName(locale);
				final String normalizedLocale = locales.get(locale);
				if (localName != null && normalizedLocale != null)
				{
					final LocalizedName lname = new LocalizedName(normalizedLocale, localName);
					names.add(lname);
				}
			}
			catch (final Exception ex)
			{
				//LOG.debug(ex.getMessage());
				continue;
			}

		}
		tag.setNames(names.isEmpty() ? null : names);
		return tag;
	}

	private ProductTag getProductTag(final ProductModel model, final Map<Locale, String> locales)
	{
		return getProductTag(model, locales, false);
	}

	private ProductTag getProductTag(final ProductModel model, final Map<Locale, String> locales, final boolean isFamilyEnabled)
	{
		final ProductTag tag = new ProductTag();
		final String name = model.getName();
		tag.setName(name == null || name.isEmpty() ? model.getCode() : name);

		tag.setExternalId(bvExportXmlUtils.getProductExternalId(model, cmsSiteModel));

		//Localized brand names
		final List<LocalizedName> names = new ArrayList<LocalizedName>();
		for (final Locale locale : locales.keySet())
		{
			try
			{
				final String localName = model.getName(locale);
				final String normalizedLocale = locales.get(locale);
				if (localName != null && normalizedLocale != null)
				{
					final LocalizedName lname = new LocalizedName(normalizedLocale, localName);
					names.add(lname);
				}
			}
			catch (final Exception ex)
			{
				//LOG.debug(ex.getMessage());
				continue;
			}
		}
		tag.setNames(names.isEmpty() ? null : names);

		String brandExternalId = null;
		String categoryExternalId = null;
		final Collection<CategoryModel> superCategories = model.getSupercategories();
		for (final CategoryModel category : superCategories)
		{
			if (brands.contains(category))
			{
				brandExternalId = StringEscapeUtils.escapeXml(ReplaceUnsupportedCharacters(category.getCode()));
			}
			else
			{
				categoryExternalId = StringEscapeUtils.escapeXml(ReplaceUnsupportedCharacters(category.getCode()));
			}
		}
		tag.setBrandExternalId((brandExternalId==null || StringEscapeUtils.escapeXml(brandExternalId).isEmpty()) ? null
				: StringEscapeUtils.escapeXml(brandExternalId));

		if (cmsSiteModel.getBvConfig().getStoreLevelCategoriesEnabled()){
			tag.setCategoryExternalId((categoryExternalId==null || StringEscapeUtils.escapeXml(categoryExternalId).isEmpty())? null
					: cmsSiteModel.getUid() + "-" + StringEscapeUtils.escapeXml(categoryExternalId));
		} else {
			tag.setCategoryExternalId((categoryExternalId==null || StringEscapeUtils.escapeXml(categoryExternalId).isEmpty())? null
					: StringEscapeUtils.escapeXml(categoryExternalId));
		}
		//		final List<ProductReferenceModel> refs = (List<ProductReferenceModel>) model.getProductReferences();
		//		final String productPageUrl = (refs != null && !refs.isEmpty()) ? refs.get(0).toString() : null;
		//		tag.setProductPageUrl(normalizeText(host_url + productModelUrlResolver.resolve(model)));

		//Localized product page urls
		final Collection<String> availableValues = languageAttributeManager.getAllAvailableValues();
		final List<LocalizedName> productPageUrls = new ArrayList<LocalizedName>();
		//		LOG.debug("******************************************************");
		for (final String value : availableValues)
		{
			//			LOG.debug("languageAttributeManager value: " + value);
			try
			{
				final String normalizedLocale = locales.get(Locale.forLanguageTag(value));
				final String localizedProductPageUrl = siteBaseUrlResolutionService.getWebsiteUrlForSite(cmsSiteModel, "/" + value,
						false, productModelUrlResolver.resolve(model));
				if (localizedProductPageUrl != null && normalizedLocale != null)
				{
					final LocalizedName localizedUrl = new LocalizedName(normalizedLocale, localizedProductPageUrl);
					//LOG.debug("added localized productPageUrl: " + localizedProductPageUrl);
					productPageUrls.add(localizedUrl);
				}
			}
			catch (final Exception ex)
			{
				//LOG.debug(ex.getMessage());
				continue;
			}
		}
		tag.setProductPageUrls(productPageUrls.isEmpty() ? null : productPageUrls);

		final String productPageUrl = siteBaseUrlResolutionService.getWebsiteUrlForSite(cmsSiteModel, false,
				productModelUrlResolver.resolve(model));
		tag.setProductPageUrl((productPageUrl==null || productPageUrl.isEmpty())? null : productPageUrl);

		//CatalogVersionModel catalogVersionModel = model.getCatalogVersion();

		String imageUrlString = bvExportXmlUtils.getImageUrl(model, cmsSiteModel);
		tag.setImageUrl((imageUrlString==null || imageUrlString.isEmpty())? null : imageUrlString);

		tag.setEans(bvExportXmlUtils.getEans(cmsSiteModel, model, isFamilyEnabled));

        final List<String> manufacturerPartNumbers = new ArrayList<String>();
        if(model.getManufacturerAID()!=null) {
            manufacturerPartNumbers.add(StringEscapeUtils.escapeXml(model.getManufacturerAID()));
        }
        tag.setManufacturerPartNumbers(manufacturerPartNumbers.isEmpty() ? null : manufacturerPartNumbers);

		final List<String> upcs = bvExportXmlUtils.getUpcs(cmsSiteModel, model, isFamilyEnabled, upcMethodName);
		tag.setUpcs(upcs.isEmpty() ? null : upcs);
		//TODO
		//		final String isbn = "unknown yet";
		//tag.setIsbn(isbn);

		String description = model.getDescription();
		if (description == null || description.isEmpty())
		{
			description = brandExternalId != null ? brandExternalId + " brand" : "empty description";
		}
		tag.setDescription((description==null || description.isEmpty())? null: description);

		final List<Description> descriptions = new ArrayList<Description>();
		for (final Locale loc : locales.keySet())
		{
			try
			{
				final String locName = model.getDescription(loc);
				final String normalizedLocale = locales.get(loc);
				if (locName != null && normalizedLocale != null)
				{
					final Description lname = new Description(normalizedLocale, locName);
					descriptions.add(lname);
				}
			}
			catch (final Exception ex)
			{
				continue;
			}
		}
		tag.setDescriptions(descriptions.isEmpty() ? null : descriptions);

		final List<String> isbns = new ArrayList<String>();
		tag.setIsbns(isbns.isEmpty() ? null : isbns);

		final List<String> modelNumbers = new ArrayList<String>();
		tag.setModelNumbers(modelNumbers.isEmpty() ? null : modelNumbers);

		final List<ProductDetailComponentModel> details = model.getProductDetailComponents();
		final List<Attribute> attributes = new ArrayList<>();
		for (final ProductDetailComponentModel detail : details)
		{
			attributes.add(new Attribute(detail.getName(), detail.getProduct().getDescription()));
		}

		// product families
		if (isFamilyEnabled && model instanceof VariantProductModel) {
			// final List<String> list = new ArrayList<String>(); // old implementation
			ProductModel productModel = model;
			while (productModel instanceof VariantProductModel)
			{
				// list.add(productModel.getCode().replace(" ", "_")); // old implementation
				productModel = ((VariantProductModel) productModel).getBaseProduct();
			}
			final String familyName = productModel.getCode().replace(" ", "_");

			// Old implementation
			/*
			 * for (int rate = 0; rate < list.size(); rate++) { final StringBuilder value = new StringBuilder(); final int
			 * controlValue = list.size() - 1 - rate; for (int i = controlValue; i >= 0; i--) { if (list.size() > 1 && i !=
			 * controlValue) { value.append("_"); } value.append(list.get(i)); } attributes.add(new
			 * Attribute(BV_FAMILY_NAME_ATTRIBUTE_ID, value.toString())); }
			 */
			attributes.add(new Attribute(BV_FAMILY_NAME_ATTRIBUTE_ID, familyName));
			attributes.add(new Attribute(BV_FAMILY_EXPAND, BV_FAMILY_NAME_ATTRIBUTE_ID + ":" + familyName));
		}

		tag.setAttributes(attributes.isEmpty() ? null : attributes);
		return tag;
	}

	public String generateTodayFilename(final String filePrefix, final String dateNumber)
	{

		final String name = (filePrefix == null ? "product_feed" : filePrefix) + fileCounter + "_" + dateNumber;
		fileCounter++;
		return name;
	}

    private void addParentCategory(List<CategoryTag> categoryTags){
        CategoryTag parentCategory = new CategoryTag();
        parentCategory.setExternalId(cmsSiteModel.getUid());
        categoryTags.add(parentCategory);
    }

    private void modifyParentCategory(List<CategoryTag> categoryTags){
        CategoryTag parentCategory = categoryTags.get(0);
        CategoryTag previousParentCategory = categoryTags.get(1);
        previousParentCategory.setParentExternalId(parentCategory.getExternalId());
    }

}
